﻿using System;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 데이터 테이블 가상 소스 속성 설명자
    /// </summary>
    public class DataTableVirtualSourcePropertyDescriptor : PropertyDescriptor
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 속성명
        /// </summary>
        private string propertyName;

        /// <summary>
        /// 속성 타입
        /// </summary>
        private Type propertyType;

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        private bool isReadOnly;

        /// <summary>
        /// 가상 소스
        /// </summary>
        private DataTableVirtualSource virtualSource;

        /// <summary>
        /// 인덱스
        /// </summary>
        private int index;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 컴포넌트 타입 - ComponentType

        /// <summary>
        /// 컴포넌트 타입
        /// </summary>
        public override Type ComponentType
        {
            get
            {
                return typeof(DataTableVirtualSource);
            }
        }

        #endregion
        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public override string Name
        {
            get
            {
                return this.propertyName;
            }
        }

        #endregion
        #region 속성 타입 - PropertyType

        /// <summary>
        /// 속성 타입
        /// </summary>
        public override Type PropertyType
        {
            get
            {
                return this.propertyType;
            }
        }

        #endregion
        #region 읽기 전용 여부 - IsReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        public override bool IsReadOnly
        {
            get
            {
                return this.isReadOnly;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataTableVirtualSourcePropertyDescriptor(virtuslSource, index, propertyName, propertyType, isReadOnly)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="virtuslSource">가상 소스</param>
        /// <param name="index">인덱스</param>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="isReadOnly">읽기 전용 여부</param>
        public DataTableVirtualSourcePropertyDescriptor(DataTableVirtualSource virtuslSource, int index, string propertyName, Type propertyType, bool isReadOnly) : base(propertyName, null)
        {
            this.virtualSource = virtuslSource;
            this.index         = index;
            this.propertyName  = propertyName;
            this.propertyType  = propertyType;
            this.isReadOnly    = isReadOnly;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 리셋 가능 여부 구하기 - CanResetValue(component)

        /// <summary>
        /// 값 리셋 가능 여부 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값 리셋 가능 여부</returns>
        public override bool CanResetValue(object component)
        {
            return false;
        }

        #endregion
        #region 값 리셋하기 - ResetValue(component)

        /// <summary>
        /// 값 리셋하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public override void ResetValue(object component)
        {
        }

        #endregion
        #region 값 설정하기 - SetValue(component, value)

        /// <summary>
        /// 값 설정하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <param name="value">값</param>
        public override void SetValue(object component, object value)
        {
            this.virtualSource.SetRowValue(component, this.index, value);
        }

        #endregion
        #region 값 구하기 - GetValue(component)

        /// <summary>
        /// 값 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값</returns>
        public override object GetValue(object component)
        {
            return this.virtualSource.GetRowValue(component, this.index);
        }

        #endregion
        #region 값 직렬화 필수 여부 구하기 - ShouldSerializeValue(component)

        /// <summary>
        /// 값 직렬화 필수 여부 구하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        /// <returns>값 직렬화 필수 여부</returns>
        public override bool ShouldSerializeValue(object component)
        {
            return true;
        }

        #endregion
    }
}